#########################
# Nagy, S., Gijbels, I., Hlubinka, D.: 
# Depth-Based Recognition of Shape Outlying Fucntions
# 07/02/2017
#
# Complete simultation study presented in the paper, and the supplementary document
#
# It is necessary to have installed the R package "depth.fd"
# this can be downloaded from 
# https://bitbucket.org/StanislavNagy/depth.fd_public
#
# To run the last simulation study in the script, download R source files 
# - OutGram.R
# - OutGramAdj.R
# from the supplementary material of Arribas-Gil, Romo (2014). Biostatics
# https://academic.oup.com/biostatistics/article-lookup/doi/10.1093/biostatistics/kxu006
# and place these two files to folder "files_depth.fd" in the working directory
# all the resulting tex tables, RData files, and eps figures are then saved in this dir
#########################

rm(list=ls(all=TRUE))
set.seed(20170204)

if(!dir.exists("files_depth.fd")) dir.create("files_depth.fd")

library(depth.fd)			
library(mvtnorm)
library(fda)									# functional principal components
library(plyr)									# rounding for Ex. S.6
library(doParallel)           						# parallel computing
nCores = detectCores()
library(abind)
acomb3 <- function(...) abind(..., along=3)

DepthAnalyse = function(X,Y,deriv=NULL,range=c(0,1)){
# Depth analyse for paper, we use 18 depths + 2 PC depths + 2 3D depths
# deriv: NULL if the default choice of the derivatives is OK
# for infimal depths provides also the adjusted infimal rank, allowing to rank 
# functions also for infimal dephts efficiently

	nD = 22									# no of depths to compute
	n = nrow(Y)									# no of functions for evaluation
	D = matrix(ncol=nD,nrow=n)						# depths
	A = matrix(ncol=nD,nrow=n)						# infimal areas
	iD = 0									# index of current depth
	#	depths without derivatives
	FM1D = depth.fd1(Y,X)
	D[,iD<-iD+1] = FM1D$Half_FD						# 1: integrated depth
	D[,iD<-iD+1] = FM1D$Half_ID						# 2: infimal depth
	A[,iD] = FM1D$Half_IA
	D[,iD<-iD+1] = depth.hM2(Y,X)$hM_norm				# 3: modal depth, scaled into [0,1]
	D[,iD<-iD+1] = depth.BD(Y,X)						# 4: band depth
	iDBD = iD
	D[,iD<-iD+1] = depth.HR(Y,X)						# 5: halfregion depth
	iDHR = iD
	RP1 = depth.RP1(Y,X,50,15)						# 	50 projections
	D[,iD<-iD+1] = RP1$Half_FD						# 6: random projection depth
	D[,iD<-iD+1] = RP1$RHalf_FD						# 7: random Tukey depth
	# 	depth based on principal components
	t = seq(0,1,len=ncol(Y))
	fX = Data2fd(argvals=t,y=t(X))
	pcX = pca.fd(fX)
	scoY = inprod(Data2fd(argvals=t,y=t(Y)),pcX$harmonics)
	D1 = depth.sample(scoY[,1],pcX$scores[,1])
	D[,iD<-iD+1] = D1$Half							# 8: PCA depth 1
	D2 = depth.sample(scoY,pcX$scores)
	D[,iD<-iD+1] = D2$Half							# 9: PCA depth 2
	#	depths with derivatives
	XK = derivatives.est(X)
	if (is.null(deriv)) YK = derivatives.est(Y,range=range) else {
	if (deriv=="Special3EX"){
		YK = derivatives.est(Y)	# X functions are diffed normally
		YK[n,1:50,2] = D1ss(t[1:50],Y[n,1:50])/(t[50]-t[1])
		YK[n,51:101,2] = D1ss(t[51:101],Y[n,51:101])/(t[101]-t[51])
		}
	if (deriv=="Special6EX"){
		YK = derivatives.est(Y)	# X functions are diffed normally
		YK[n,,2] = 0		# Y function is piecewise constant
		}
		}
	FM2D = depth.fd2(YK,XK)
	D[,iD<-iD+1] = FM2D$Half_FD						# 10: integrated depth
	D[,iD<-iD+1] = FM2D$Half_ID						# 11: infimal depth
	A[,iD] = FM2D$Half_IA
	D[,iD<-iD+1] = depth.hM2(YK,XK)$hM_norm				# 12: modal depth, scaled into [0,1]
	D[,iD<-iD+1] = D[,iDBD] + depth.BD(YK[,,2],XK[,,2])		# 13: band depth
	D[,iD<-iD+1] = D[,iDHR] + depth.HR(YK[,,2],XK[,,2])		# 14: halfregion depth
	RP2 = depth.RP2(YK,XK,50)						# 	50 projections
	D[,iD<-iD+1] = RP2$Half_FD						# 15: random projection depth (R2 -> HD)
	D[,iD<-iD+1] = RP2$Half_DD						# 16: random projection depth (R2 -> R1 -> HD)							
	#	higher order depths
	DiD = depth.fd1(Y,X,order=2)
	DiDA = depth.fd1(Y,X,order=2,approx=100)
	DiDT = depth.fd1(Y,X,order=3,approx=100)
	D[,iD<-iD+1] = DiD$Half_FD						# 17: 2nd order integrated						
	D[,iD<-iD+1] = DiDA$Half_FD						# 18: 2nd order integrated approximated	
	D[,iD<-iD+1] = DiDT$Half_FD						# 19: 3rd order integrated approximated
	#
	D[,iD<-iD+1] = DiD$Half_ID						# 20: 2nd order infimal
	A[,iD] = DiD$Half_IA						
	D[,iD<-iD+1] = DiDA$Half_ID						# 21: 2nd order infimal approximated
	A[,iD] = DiDA$Half_IA						
	D[,iD<-iD+1] = DiDT$Half_ID						# 22: 3rd order infimal approximated
	A[,iD] = DiDT$Half_IA						

	colnames(D) = DepthNames
	return(list(D=D,A=A))
}      
      
DepthNames = 	c(	"FD","ID","hM","BD","HR","RP","RF","PC","PCB",
				"FDB","IDB","hMB","BDB","HRB","RPB","RPD",
				"FDD","FDA","FDT",
				"IDD","IDA","IDT")     

MinimalProcessing = function(DT){
# function processing results for one run of the study
	D = DT[,1:nD,]
	A = DT[,(nD+1):(2*nD),]
	#
	RT = apply(-D,2:3,rank,ties.method="max")			# ranking
	M2 = apply(RT[dim(RT)[1],,],1,mean)				# mean depth rank
	S2 = apply(RT[dim(RT)[1],,],1,sd)				# sd of depth rank
	TT = apply(D,2:3,function(x) sum(x==min(x)))		# no of minimal value ties
	M3 = apply(TT,1,mean)						# mean nr of ties
	S3 = apply(TT,1,sd)						# sd of nr of ties
	#	infimal ranks
	B = dim(A)[3]
	n = dim(A)[1]
	AT = array(dim=c(n,nD,B))					# infimal rank
	ATT = array(dim=c(nD,B))					# no of minimal value ties
	for(i in 1:nD) for(b in 1:B){
		if(!is.na(A[1,i,b])){
			AT[,i,b] = infimalRank(D[,i,b],A[,i,b])
			ATT[i,b] = sum(AT[,i,b]==min(AT[,i,b]))
			}
		}	
	M4 = apply(AT[dim(AT)[1],,],1,mean)				# mean infimal depth rank
	S4 = apply(AT[dim(AT)[1],,],1,sd)				# sd of infimal depth rank
	M5 = apply(ATT,1,mean)						# mean nr of ties
	S5 = apply(ATT,1,sd)						# sd of nr of ties
	#
	print(cbind(cbind(M2,S2,M3,S3),cbind(M4,S4,M5,S5)))
	}

stopurl = "https://cran.r-project.org/"
Tstart = Sys.time()

#####################
#	start of simulation
#####################

eval = 101
t = seq(0,1,len=eval)
n = 200				# sample size
nD = length(DepthNames)		# no of used depths
B = 100				# no of simulations
X = matrix(ncol=eval,nrow=n)
Y = matrix(ncol=eval,nrow=n)	

##############################
# motivating examples, ex. 1: smooth functions
##############################
cl <- makePSOCKcluster(nCores)
registerDoParallel(cl)
DT = foreach(b=1:B, .combine='acomb3', .multicombine=TRUE, .packages=c("depth.fd","fda")) %dopar% {
      I = rnorm(n,0,2) 			
      S = rexp(n)
      for(i in 1:n) X[i,] = I[i]+S[i]*atan(t)
      I = rnorm(n,0,2) 			
      S = rexp(n)
      for(i in 1:n) Y[i,] = I[i]+S[i]*atan(t)
      IO = 1	
      SO = -2
      O = IO + SO*atan(t)
      D = DepthAnalyse(X,rbind(Y,O))
	DG = cbind(D$D,D$A)
      DG
	}
stopCluster(cl)

MinimalProcessing(DT)

dimnames(DT)[[1]] = c(paste("fun_",1:n,sep=""),"outlier")
dimnames(DT)[[2]] = rep(DepthNames,2)
dimnames(DT)[[3]] = paste("rep_",1:B,sep="")

save(DT,file="files_depth.fd/SimulationStudy_Ex1.RData")

##########################
# motivating examples, ex. 2: gaussian processes
##########################
cl <- makePSOCKcluster(nCores)
registerDoParallel(cl)
DT = foreach(b=1:B, .combine='acomb3', .multicombine=TRUE, .packages=c("mvtnorm","depth.fd","fda")) %dopar% {
      C = function(t) return(.2*exp(-abs(outer(t,t,"-"))/.3))
      I = rnorm(n,0,2) 
      S = rexp(n)
      for(i in 1:n) X[i,] = I[i]+S[i]*atan(t)+rmvnorm(1,rep(0,eval),C(t))
      I = rnorm(n,0,2) 
      S = rexp(n)
      for(i in 1:n) Y[i,] = I[i]+S[i]*atan(t)+rmvnorm(1,rep(0,eval),C(t))
      IO = 1
      SO = -2
      O = IO + SO*atan(t)+rmvnorm(1,rep(0,eval),C(t))
      D = DepthAnalyse(X,rbind(Y,O))
	DG = cbind(D$D,D$A)
      DG
	}
stopCluster(cl)

MinimalProcessing(DT)

dimnames(DT)[[1]] = c(paste("fun_",1:n,sep=""),"outlier")
dimnames(DT)[[2]] = rep(DepthNames,2)
dimnames(DT)[[3]] = paste("rep_",1:B,sep="")

save(DT,file="files_depth.fd/SimulationStudy_Ex2.RData")

############################
# motivating examples, ex. 3: discontinuous function
############################
cl <- makePSOCKcluster(nCores)
registerDoParallel(cl)
DT = foreach(b=1:B, .combine='acomb3', .multicombine=TRUE, .packages=c("depth.fd","fda")) %dopar% {
      I = rnorm(n,0,2) 
      S = rexp(n)
      for(i in 1:n) X[i,] = I[i]+S[i]*atan(t)
      I = rnorm(n,0,2) 
      S = rexp(n)
      for(i in 1:n) Y[i,] = I[i]+S[i]*atan(t)
      IO1 = .5
      SO1 = -log(.5)
      IO2 = -.5
      SO2 = -log(.5)
      O = (IO1 + SO1*atan(t))*(t<.5) + (IO2 + SO2*atan(t))*(t>=.5)
      D = DepthAnalyse(X,rbind(Y,O),deriv="Special3EX")
	DG = cbind(D$D,D$A)
      DG
	}
stopCluster(cl)

MinimalProcessing(DT)

dimnames(DT)[[1]] = c(paste("fun_",1:n,sep=""),"outlier")
dimnames(DT)[[2]] = rep(DepthNames,2)
dimnames(DT)[[3]] = paste("rep_",1:B,sep="")

save(DT,file="files_depth.fd/SimulationStudy_Ex3.RData")

############################
# motivating examples, ex. 4: convex-concave function
############################
cl <- makePSOCKcluster(nCores)
registerDoParallel(cl)
DT = foreach(b=1:B, .combine='acomb3', .multicombine=TRUE, .packages=c("depth.fd","fda")) %dopar% {
      I = rnorm(n,0,2) 
      S = rexp(n)
      for(i in 1:n) X[i,] = I[i]+S[i]*atan(t)
      I = rnorm(n,0,2) 
      S = rexp(n)
      for(i in 1:n) Y[i,] = I[i]+S[i]*atan(t)
      a = 4
      O = tan(t)/a
      D = DepthAnalyse(X,rbind(Y,O))
	DG = cbind(D$D,D$A)
      DG
	}
stopCluster(cl)

MinimalProcessing(DT)

dimnames(DT)[[1]] = c(paste("fun_",1:n,sep=""),"outlier")
dimnames(DT)[[2]] = rep(DepthNames,2)
dimnames(DT)[[3]] = paste("rep_",1:B,sep="")

save(DT,file="files_depth.fd/SimulationStudy_Ex4.RData")

############################
# motivating examples, ex. 5: periodic outlier
############################
cl <- makePSOCKcluster(nCores)
registerDoParallel(cl)
DT = foreach(b=1:B, .combine='acomb3', .multicombine=TRUE, .packages=c("depth.fd","fda")) %dopar% {
	f = Vectorize(function(t){	# basis functions
	return(c(sin(t),sin(2*t)))
	})
	x = Vectorize(function(t) 
	return(2*sin(5*t)))

	t = seq(-pi,pi,len=eval)
      C = matrix(rnorm(2*n,sd=.3),ncol=2)
      for(i in 1:n) X[i,] = C[i,]%*%f(t)+rnorm(1)
      C = matrix(rnorm(2*n,sd=.3),ncol=2)
      for(i in 1:n) Y[i,] = C[i,]%*%f(t)+rnorm(1)
      D = DepthAnalyse(X,rbind(Y,x(t)))
	DG = cbind(D$D,D$A)
      DG
	}
stopCluster(cl)

MinimalProcessing(DT)

dimnames(DT)[[1]] = c(paste("fun_",1:n,sep=""),"outlier")
dimnames(DT)[[2]] = rep(DepthNames,2)
dimnames(DT)[[3]] = paste("rep_",1:B,sep="")

save(DT,file="files_depth.fd/SimulationStudy_Ex5.RData")

############################
# motivating examples, ex. 6: crude periodic outlier
############################
cl <- makePSOCKcluster(nCores)
registerDoParallel(cl)
DT = foreach(b=1:B, .combine='acomb3', .multicombine=TRUE, .packages=c("depth.fd","fda","plyr")) %dopar% {
	f = Vectorize(function(t){	# basis functions
	return(c(sin(t),sin(2*t)))
	})
	x = Vectorize(function(t) 
	return(round_any(2*sin(5*t),0.1)))

	t = seq(-pi,pi,len=eval)
      C = matrix(rnorm(2*n,sd=.3),ncol=2)
      for(i in 1:n) X[i,] = C[i,]%*%f(t)+rnorm(1)
      C = matrix(rnorm(2*n,sd=.3),ncol=2)
      for(i in 1:n) Y[i,] = C[i,]%*%f(t)+rnorm(1)
      D = DepthAnalyse(X,rbind(Y,x(t)),deriv="Special6EX")
	DG = cbind(D$D,D$A)
      DG
	}
stopCluster(cl)

MinimalProcessing(DT)

dimnames(DT)[[1]] = c(paste("fun_",1:n,sep=""),"outlier")
dimnames(DT)[[2]] = rep(DepthNames,2)
dimnames(DT)[[3]] = paste("rep_",1:B,sep="")

save(DT,file="files_depth.fd/SimulationStudy_Ex6.RData")

######
# End of the main simulation study
######

Tend = Sys.time()
shell.exec(stopurl)

print(paste("Simulation study started: ", Tstart,sep=""))
print(paste("Simulation study finished: ", Tend,sep=""))

#########################
#
# Processing of the results of the simulation study
#
#########################

rm(list=ls(all=TRUE))

if(!dir.exists("files_depth.fd")) warning("Check working directory")

library(abind)
library(xtable)
library(depth.fd)

ExNames = c("_Ex1","_Ex2","_Ex3","_Ex4","_Ex5","_Ex6")
DepthNames = 	paste("\\",c(	
				"FD","ID","hM","BD","HR","RP","RF","PC","PCB",
				"FDB","IDB","hMB","BDB","HRB","RPB","RPD",
				"FDD","FDA","FDT",
				"IDD","IDA","IDT"
				),sep="")
nD = length(DepthNames)
nE = length(ExNames)

for(i in 1:nE){
	load(paste("files_depth.fd/SimulationStudy",ExNames[i],".RData",sep=""))
	D = DT
	if(i==1) DG = D else DG = abind(DG,D,along=4)
	}
B = dim(DG)[3]
DC = DG[,1:nD,,]							# only depths, no infimal area indices
AC = DG[,(nD+1):(2*nD),,]					# only infimal area indices

DC = aperm(DC,c(1,2,4,3))
#
AC = aperm(AC,c(1,2,4,3))

RT = apply(-DC,2:4,rank,ties.method="max")		# ranking
TT = apply(DC,2:4,function(x) sum(x==min(x)))		# no of minimal value ties
#
AT = array(dim=dim(AC))						# infimal rank
ATT = array(dim=dim(AC)[-1])					# no of minimal value ties
for(i in 1:nD) for(b in 1:B) for(e in 1:nE) {
		if(!is.na(AC[1,i,e,b])){
			AT[,i,e,b] = infimalRank(DC[,i,e,b],AC[,i,e,b])
			ATT[i,e,b] = sum(AT[,i,e,b]==min(AT[,i,e,b]))
			}
		}	

M2 = apply(RT[dim(RT)[1],,,],1:2,mean)			# mean depth rank
S2 = apply(RT[dim(RT)[1],,,],1:2,sd)			# sd of depth rank
C2 = paste(sprintf("%.3f", round(M2,3))," (",sprintf("%.1f", round(S2,3)),")",sep="")
#
M4 = apply(AT[dim(AT)[1],,,],1:2,mean)			# mean depth rank
S4 = apply(AT[dim(AT)[1],,,],1:2,sd)			# sd of depth rank
C4 = paste(sprintf("%.3f", round(M4,3))," (",sprintf("%.1f", round(S4,3)),")",sep="")

M3 = apply(TT,1:2,mean)						# mean nr of ties
S3 = apply(TT,1:2,sd)						# sd of nr of ties
C3 = paste(sprintf("%.3f", round(M3,3))," (",sprintf("%.1f", round(S3,3)),")",sep="")
#
M5 = apply(ATT,1:2,mean)						# mean nr of ties
S5 = apply(ATT,1:2,sd)							# sd of nr of ties
C5 = paste(sprintf("%.3f", round(M5,3))," (",sprintf("%.1f", round(S5,3)),")",sep="")

fillTAB = function(jmin){
	TAB = matrix(nrow=nD,ncol=(2*2+1))
	TAB[,1] = DepthNames
	for(i in 1:nD) for(j in jmin:(jmin+1)){
		TAB[i,1+(j-jmin)*2+1] = paste(sprintf("%.1f", round(M2[i,j],3))," \\,(",sprintf("%04.1f", round(S2[i,j],3)),")",sep="")
		TAB[i,1+(j-jmin)*2+2] = paste(sprintf("%.1f", round(M3[i,j],3))," \\,(",sprintf("%.1f", round(S3[i,j],3)),")",sep="")
		}
	rownames(TAB) = 1:nD
	colnames(TAB) = c("Abbr",rep(c("Rank","Ties"),2))
	return(TAB)
	}

fillTAB_inf = function(jmin){
	I = !is.na(M4[,1])
	M4 = M4[I,]
	S4 = S4[I,]
	M5 = M5[I,]
	S5 = S5[I,]
	TAB = matrix(nrow=sum(I),ncol=(2*2+1))
	TAB[,1] = DepthNames[I]
	for(i in 1:sum(I)) for(j in jmin:(jmin+1)){
		TAB[i,1+(j-jmin)*2+1] = paste(sprintf("%.1f", round(M4[i,j],3))," \\,(",sprintf("%04.1f", round(S4[i,j],3)),")",sep="")
		TAB[i,1+(j-jmin)*2+2] = paste(sprintf("%.1f", round(M5[i,j],3))," \\,(",sprintf("%.1f", round(S5[i,j],3)),")",sep="")
		}
	rownames(TAB) = (1:nD)[I]
	colnames(TAB) = c("Abbr",rep(c("Rank","Ties"),2))
	return(TAB)
	}

# first two examples: table

TAB = fillTAB(1)

xTAB = xtable(TAB,label="table:results1",caption = "The table of results of the simulation study in Examples \\ref{ex:smooth} and \\ref{ex:Gaussian}.")
align(xTAB) = "c|l|cc|cc"
xTABp = print(xTAB,
	hline.after=c(0,nrow(xTAB)),
	add.to.row = list(list(-1,0,9,16),
	c(" & & \\multicolumn{2}{c}{Example \\ref{ex:smooth}} & \\multicolumn{2}{|c}{Example \\ref{ex:Gaussian}} \\\\ \n no",
	"",
	rep("\\hdashline \n",2))),	
	sanitize.text.function=function(x) paste("$",x,"$",sep=""),
	sanitize.colnames.function=function(x)x,
	caption.placement = "top") 
write(xTABp, file = "files_depth.fd/ResTable1.tex")

TAB = fillTAB_inf(1)

xTAB = xtable(TAB,label="table:results_inf1",caption = "The table of results of the simulation study in Examples \\ref{ex:smooth} and \\ref{ex:Gaussian}: Infimal depth ranks.")
align(xTAB) = "c|l|cc|cc"
xTABp = print(xTAB,
	hline.after=c(0,nrow(xTAB)),
	add.to.row = list(list(-1,0,1,2),
	c(" & & \\multicolumn{2}{c}{Example \\ref{ex:smooth}} & \\multicolumn{2}{|c}{Example \\ref{ex:Gaussian}} \\\\ \n no",
	"",
	rep("\\hdashline \n",2))),	
	sanitize.text.function=function(x) paste("$",x,"$",sep=""),
	sanitize.colnames.function=function(x)x,
	caption.placement = "top") 
write(xTABp, file = "files_depth.fd/ResTableInf1.tex")

# second two examples: table

TAB = fillTAB(3)

xTAB = xtable(TAB,label="table:results2",caption = "The table of results of the simulation study in Examples \\ref{ex:broken} and \\ref{ex:convex}.")
align(xTAB) = "c|l|cc|cc"
xTABp = print(xTAB,
	hline.after=c(0,nrow(xTAB)),
	add.to.row = list(list(-1,0,9,16),
	c(" & & \\multicolumn{2}{c}{Example \\ref{ex:broken}} & \\multicolumn{2}{|c}{Example \\ref{ex:convex}} \\\\ \n no",
	"",
	rep("\\hdashline \n",2))),
	sanitize.text.function=function(x) paste("$",x,"$",sep=""),
	sanitize.colnames.function=function(x)x,
	caption.placement = "top") 
write(xTABp, file = "files_depth.fd/ResTable2.tex")

TAB = fillTAB_inf(3)

xTAB = xtable(TAB,label="table:results_inf2",caption = "The table of results of the simulation study in Examples \\ref{ex:broken} and \\ref{ex:convex}: Infimal depth ranks.")
align(xTAB) = "c|l|cc|cc"
xTABp = print(xTAB,
	hline.after=c(0,nrow(xTAB)),
	add.to.row = list(list(-1,0,1,2),
	c(" & & \\multicolumn{2}{c}{Example \\ref{ex:broken}} & \\multicolumn{2}{|c}{Example \\ref{ex:convex}} \\\\ \n no",
	"",
	rep("\\hdashline \n",2))),
	sanitize.text.function=function(x) paste("$",x,"$",sep=""),
	sanitize.colnames.function=function(x)x,
	caption.placement = "top") 
write(xTABp, file = "files_depth.fd/ResTableInf2.tex")

# last two examples: table

TAB = fillTAB(5)

xTAB = xtable(TAB,label="table:results3",caption = "The table of results of the simulation study in Examples \\ref{ex:periodic} and \\ref{ex:crude periodic}.")
align(xTAB) = "c|l|cc|cc"
xTABp = print(xTAB,
	hline.after=c(0,nrow(xTAB)),
	add.to.row = list(list(-1,0,9,16),
	c(" & & \\multicolumn{2}{c}{Example \\ref{ex:periodic}} & \\multicolumn{2}{|c}{Example \\ref{ex:crude periodic}} \\\\ \n no",
	"",
	rep("\\hdashline \n",2))),
	sanitize.text.function=function(x) paste("$",x,"$",sep=""),
	sanitize.colnames.function=function(x)x,
	caption.placement = "top") 
write(xTABp, file = "files_depth.fd/ResTable3.tex")

TAB = fillTAB_inf(5)

xTAB = xtable(TAB,label="table:results_inf3",caption = "The table of results of the simulation study in Examples \\ref{ex:periodic} and \\ref{ex:crude periodic}: Infimal depth ranks.")
align(xTAB) = "c|l|cc|cc"
xTABp = print(xTAB,
	hline.after=c(0,nrow(xTAB)),
	add.to.row = list(list(-1,0,1,2),
	c(" & & \\multicolumn{2}{c}{Example \\ref{ex:periodic}} & \\multicolumn{2}{|c}{Example \\ref{ex:crude periodic}} \\\\ \n no",
	"",
	rep("\\hdashline \n",2))),
	sanitize.text.function=function(x) paste("$",x,"$",sep=""),
	sanitize.colnames.function=function(x)x,
	caption.placement = "top") 
write(xTABp, file = "files_depth.fd/ResTableInf3.tex")

#########################
#
# Figures from the paper and the supplementary document
#
#########################

rm(list=ls(all=TRUE))
if(!dir.exists("files_depth.fd")) warning("Check working directory")

library(depth.fd)
library(mvtnorm)

cexaxis = 1.3
cexlab = 1.5
op0<-par(cex.axis=cexaxis,cex.lab=cexlab)

eval = 101
t = seq(0,1,len=eval)
n = 200				# sample size
X = matrix(ncol=eval,nrow=n)
Y = matrix(ncol=eval,nrow=n)	

###
# ex 1
###

I = rnorm(n,0,2) 			
S = rexp(n)
IO = 1	
SO = -2	
for(i in 1:n) X[i,] = I[i]+S[i]*atan(t)
O = IO + SO*atan(t)
O = matrix(O,nrow=1)

postscript("files_depth.fd/Ex1_functions.eps",height=6,width=6,horizontal=FALSE)
plot(rep(t,n),X,type="n",xlab = "X", ylab = "Y")
for(i in 1:n) lines(t,X[i,],lwd=.75,lty=3)
lines(t,O,col=2,lwd=3,lty=1)
dev.off()

postscript("files_depth.fd/Ex1_basis.eps",height=6,width=6,horizontal=FALSE)
plot(c(I,IO),c(S,SO),col=c(rep(1,n),2),pch=c(rep(1,n),17),xlab="A",ylab="B")
dev.off()

###
# ex 2
###

C = function(t) return(.2*exp(-abs(outer(t,t,"-"))/.3))
I = rnorm(n,0,2) 
S = rexp(n)
IO = 1
SO = -2
for(i in 1:n) X[i,] = I[i]+S[i]*atan(t)+rmvnorm(1,rep(0,eval),C(t))
O = IO + SO*atan(t)+rmvnorm(1,rep(0,eval),C(t))
O = matrix(O,nrow=1)

postscript("files_depth.fd/Ex2_functions.eps",height=6,width=6,horizontal=FALSE)
plot(rep(t,n),X,type="n",xlab = "X", ylab = "Y")
for(i in 1:n) lines(t,X[i,],lwd=.75,lty=3)
lines(t,O,col=2,lwd=3,lty=1)
dev.off()

postscript("files_depth.fd/Ex2_FMplot.eps",height=6,width=6,horizontal=FALSE)
shape.fd.analysis(O,X)
mtext(side = 1, "A", line = 3, cex=cexlab)
mtext(side = 2, "B", line = 3, cex=cexlab)
dev.off()

postscript("files_depth.fd/Ex2_DiffDplot.eps",height=6,width=6,horizontal=FALSE)
DDA = shape.fd.analysis(O,X,order=2)
dev.off()

# this is how D1ss works
plot(t,O,col=1,lwd=2,lty=2,type="l",xlab="X",ylab="Y")
sp <- smooth.spline(t, O)
S = smooth.spline(t, O, spar = sp$spar + 0.1384)
lines(S,col="gray",lwd=3,lty=1)
predict(S,t,deriv=1)$y
derivatives.est(O)[,,2]
DdA = diag(DDA$PHD[-1,])
DdO = rep(NA,eval)
OD = derivatives.est(O)
XD = derivatives.est(X)
for(i in 1:eval) DdO[i] = depth(OD[,i,],XD[,i,])

write(round(mean(DdO),3),"files_depth.fd/Depth1.txt")
write(round(mean(DdA),3),"files_depth.fd/Depth1B.txt")
write(min(DdO),"files_depth.fd/Depth2.txt")
write(min(DdA),"files_depth.fd/Depth2B.txt")

###
# ex 3
###

I = rnorm(n,0,2) 
S = rexp(n)
IO1 = .5
SO1 = -log(.5)
IO2 = -.5
SO2 = SO1
for(i in 1:n) X[i,] = I[i]+S[i]*atan(t)
O = (IO1 + SO1*atan(t))*(t<.5) + (IO2 + SO2*atan(t))*(t>=.5)
O = matrix(O,nrow=1)

postscript("files_depth.fd/Ex3_functions.eps",height=6,width=6,horizontal=FALSE)
plot(rep(t,n),X,type="n",xlab="X",ylab="Y")
for(i in 1:n) lines(t,X[i,],lwd=.75,lty=3)
lines(t,O,col=2,lwd=3,lty=1)
dev.off()

XB = rbind(X,O)
XKB = derivatives.est(XB)      # normal functions are diffed normally
XKB[n+1,1:50,2] = D1ss(t[1:50],O[1:50])/(t[50]-t[1])
XKB[n+1,51:101,2] = D1ss(t[51:101],O[51:101])/(t[101]-t[51])
XK = XKB[1:nrow(X),,]
D = matrix(ncol=eval,nrow=11)
for(i in 1:11) for(j in 1:eval) D[i,j]=depth(XKB[n+2-i,j,],XKB[,j,])

postscript("files_depth.fd/Ex3_FMplot2.eps",height=6,width=6,horizontal=FALSE)
plot(rep(t,11),D,type="n",ylim = c(0,max(D)),xlab="A",ylab="B")
for(i in 2:11) lines(t,D[i,],lwd=.75,lty=2)
lines(t,D[1,],col=2,lwd=3,lty=1)
dev.off()

postscript("files_depth.fd/Ex3_DiffDplot.eps",height=6,width=6,horizontal=FALSE)
shape.fd.analysis(O,X,order=2)
dev.off()

###
# ex 4
###

X = Y = matrix(nrow=n,ncol=eval)
I = rnorm(n,0,2) 
S = rexp(n)
for(i in 1:n) X[i,] = I[i]+S[i]*atan(t)
I = rnorm(n,0,2) 
S = rexp(n)
for(i in 1:n) Y[i,] = I[i]+S[i]*atan(t)
a = 4
O = tan(t)/a
O = matrix(O,nrow=1)

postscript("files_depth.fd/Ex4_functions.eps",height=6,width=6,horizontal=FALSE)
plot(rep(t,n),X,type="n",xlab="X",ylab="Y")
for(i in 1:n) lines(t,X[i,],lwd=.75,lty=3)
lines(t,O,col=2,lwd=3,lty=1)
dev.off()

XD = X
for(i in 1:n) XD[i,] = S[i]/(1+t^2)
OD = 1/cos(t)^2/a
O = matrix(O,nrow=1)

postscript("files_depth.fd/Ex4_derivatives.eps",height=6,width=6,horizontal=FALSE)
plot(rep(t,n),XD,type="n",xlab="A",ylab="B")
for(i in 1:n) lines(t,XD[i,],lwd=.75,lty=3)
lines(t,OD,col=2,lwd=3,lty=1)
dev.off()

###
# ex 5
###

f = Vectorize(function(t){	# basis functions
return(c(sin(2*pi*(t-1/2)),sin(2*2*pi*(t-1/2))))
})
x = Vectorize(function(t) 
return(2*sin(5*2*pi*(t-1/2))))

t = seq(0,1,len=eval)
C = matrix(rnorm(2*n,sd=.3),ncol=2)
for(i in 1:n) X[i,] = C[i,]%*%f(t)+rnorm(1)
O = x(t)

postscript("files_depth.fd/Ex5_functions.eps",height=6,width=6,horizontal=FALSE)
plot(rep(t,n),X,type="n",xlab="X",ylab="Y",ylim=c(-3,3))
for(i in 1:n) lines(t,X[i,],lwd=.75,lty=3)
lines(t,O,col=2,lwd=3,lty=1)
dev.off()

###
# ex 6
###

f = Vectorize(function(t){	# basis functions
return(c(sin(2*pi*(t-1/2)),sin(2*2*pi*(t-1/2))))
})
x = Vectorize(function(t) 
return(round_any(2*sin(5*2*pi*(t-1/2)),0.1)))

t = seq(0,1,len=eval)
C = matrix(rnorm(2*n,sd=.3),ncol=2)
for(i in 1:n) X[i,] = C[i,]%*%f(t)+rnorm(1)
O = x(t)

postscript("files_depth.fd/Ex6_functions.eps",height=6,width=6,horizontal=FALSE)
plot(rep(t,n),X,type="n",xlab="X",ylab="Y",ylim=c(-3,3))
for(i in 1:n) lines(t,X[i,],lwd=.75,lty=3)
lines(t,O,col=2,lwd=3,lty=1,type="s")
dev.off()

#########################
#
# Real data analysis, Section 4 of the paper
#
#########################

rm(list=ls(all=TRUE))
if(!dir.exists("files_depth.fd")) warning("Check working directory")

library(depth.fd)

cexaxis = 1.3
cexlab = 1.5
op0<-par(cex.axis=cexaxis,cex.lab=cexlab)

# to repeat the analysis performed in the paper Nagy et al. (2017), 
# choose population2010 data. To perform the analysis on the new 2015 revision of
# the data, you may change below population2010 to population, and change the Yrs
# variable accordingly

# data(population)
# Pop = population
# Yrs = 1950:2015

data(population2010)
Pop = population2010
Yrs = 1950:2010

yn = length(Yrs)
X = data.matrix(Pop[,1:yn])
rownames(X) = rownames(Pop)
X = X[(X[,31]>1000)&(X[,31]<15000),]	# consider over countries with more than million and less than 15 millions	
n = nrow(X)

set.seed(1)						# the results of the 3rd order depth are approximated, thus are random
postscript("files_depth.fd/PopData_pairs.eps",height=6,width=6,horizontal=FALSE)
O = shape.fd.outliers(X,approx=1000,print=TRUE,plotpairs=TRUE)
dev.off()

# in the analysis of population2010 data, the resulting matrix should look as follows:
# O =  matrix(FALSE, nrow=4,ncol=dim(X)[1])
# O[1,c(103,52,91,41,21,54)] = TRUE
# O[2,c(18,84,58,63)] = TRUE
# O[3,c(36,55)] = TRUE

colOut1 = "olivedrab4"
colOut2 = "navy"
colOut3 = "orangered3"
lt = c(1,2,4,5,6,7)

postscript("files_depth.fd/PopData_functions.eps",height=6,width=6)
plot(rep(Yrs,nrow(X)),X,type="n",xlab="Year",ylab="Population (in thousands)")
for(i in 1:nrow(X)) lines(Yrs,X[i,],col=1,lwd=1.5,lty=1)
dev.off()

sort(X[O[1,],1])

postscript("files_depth.fd/PopData_functions1.eps",height=6,width=6)
plot(rep(Yrs,nrow(X)),X,type="n",xlab="Year",ylab="Population (in thousands)")
for(i in 1:nrow(X)) lines(Yrs,X[i,],col=1,lwd=1.5,lty=3)
lti = 0
for(i in 1:nrow(X)){
	if(O[1,i]) lines(Yrs,X[i,],col=colOut1,lwd=3,lty=lt[lti<-lti+1])
	}
dev.off()

sort(X[O[2,],1])

postscript("files_depth.fd/PopData_functions2.eps",height=6,width=6)
plot(rep(Yrs,nrow(X)),X,type="n",xlab="Year",ylab="Population (in thousands)")
for(i in 1:nrow(X)) lines(Yrs,X[i,],col=1,lwd=1.5,lty=3)
lti = 0
for(i in 1:nrow(X)){
	if(O[2,i]) lines(Yrs,X[i,],col=colOut2,lwd=3,lty=lt[lti<-lti+1])
	}
dev.off()

sort(X[O[3,],1])
sort(X[O[4,],1])

postscript("files_depth.fd/PopData_functions3.eps",height=6,width=6)
plot(rep(Yrs,nrow(X)),X,type="n",xlab="Year",ylab="Population (in thousands)")
for(i in 1:nrow(X)) lines(Yrs,X[i,],col=1,lwd=1.5,lty=3)
lt = 0
for(i in 1:nrow(X)){
	if(O[3,i]) lines(Yrs,X[i,],col=colOut3,lwd=3,lty=lt<-lt+1)
	if(O[4,i]) lines(Yrs,X[i,],col=colOut3,lwd=3,lty=lt<-lt+1)
	}
dev.off()


##
# comparison of functions with lowest depth in the real data set
##

O =  matrix(FALSE, nrow=4,ncol=dim(X)[1])
O[1,c(103,52,91,41,21,54)] = TRUE
O[2,c(18,84,58,63)] = TRUE
O[3,c(36,55)] = TRUE

ID = matrix(nrow=n,ncol=3)
FD = ID
IA = ID
D = depth.fd1(X,X)
ID[,1] = D$Half_ID
FD[,1] = D$Half_FD
IA[,1] = D$Half_IA
D = depth.fd1(X,X,order=2)
ID[,2] = D$Half_ID
FD[,2] = D$Half_FD
IA[,2] = D$Half_IA
D = depth.fd1(X,X,order=3,approx=201)
ID[,3] = D$Half_ID
FD[,3] = D$Half_FD
IA[,3] = D$Half_IA

postscript("files_depth.fd/PopData_comparison1.eps",height=6,width=6,horizontal=FALSE)
plot(rank(-FD[,1]),infimalRank(ID[,1],IA[,1]),xlab="X1",ylab="Y1",main="1st order depth")
points(rank(-FD[,1])[O[1,]],infimalRank(ID[,1],IA[,1])[O[1,]],col=colOut1,pch=16)
legend("topleft",c("1st order outliers"),pch=16,col=c(colOut1))
dev.off()

postscript("files_depth.fd/PopData_comparison2.eps",height=6,width=6,horizontal=FALSE)
plot(rank(-FD[,2]),infimalRank(ID[,2],IA[,2]),xlab="X2",ylab="Y2",main="2nd order depth")
points(rank(-FD[,2])[O[1,]],infimalRank(ID[,2],IA[,2])[O[1,]],col=colOut1,pch=16)
points(rank(-FD[,2])[O[2,]],infimalRank(ID[,2],IA[,2])[O[2,]],col=colOut2,pch=16)
legend("topleft",c("1st order outliers","2nd order outliers"),pch=16,col=c(colOut1,colOut2))
dev.off()

postscript("files_depth.fd/PopData_comparison3.eps",height=6,width=6,horizontal=FALSE)
plot(rank(-FD[,3]),infimalRank(ID[,3],IA[,3]),xlab="X3",ylab="Y3",main="3rd order depth")
points(rank(-FD[,3])[O[1,]],infimalRank(ID[,3],IA[,3])[O[1,]],col=colOut1,pch=16)
points(rank(-FD[,3])[O[2,]],infimalRank(ID[,3],IA[,3])[O[2,]],col=colOut2,pch=16)
points(rank(-FD[,3])[O[3,]],infimalRank(ID[,3],IA[,3])[O[3,]],col=colOut3,pch=16)
legend("topleft",c("1st order outliers","2nd order outliers","3rd order outliers"),pch=16,col=c(colOut1,colOut2,colOut3))
dev.off()

#########################
#
# Tables of the computational times for the depths
#
#########################

rm(list=ls(all=TRUE))
if(!dir.exists("files_depth.fd")) warning("Check working directory")

library(depth.fd)	
library(xtable)
library(mvtnorm)
library(fda)

DepthNames = 	paste("\\",c(	
				"FD","ID","hM","BD","HR","RP","RF","PC","PCB",
				"FDB","IDB","hMB","BDB","HRB","RPB","RPD",
				"FDD","FDA","FDT",
				"IDD","IDA","IDT"
				),sep="")

eval = 101
t = seq(0,1,len=eval)
n = 200				# sample size
X = matrix(ncol=eval,nrow=n)
Y = matrix(ncol=eval,nrow=n)	
C = function(t) return(.2*exp(-abs(outer(t,t,"-"))/.3))
I = rnorm(n,0,2) 
S = rexp(n)
for(i in 1:n) X[i,] = I[i]+S[i]*atan(t)+rmvnorm(1,rep(0,eval),C(t))
I = rnorm(n,0,2) 
S = rexp(n)
for(i in 1:n) Y[i,] = I[i]+S[i]*atan(t)+rmvnorm(1,rep(0,eval),C(t))

###
T = rep(NA,nD<-length(DepthNames))
w = 3

T[1] = system.time(depth.fd1(Y,X))[w]
T[2] = T[1]
T[3] = system.time(depth.hM2(Y,X))[w]
T[4] = system.time(depth.BD(Y,X))[w]
T[5] = system.time(depth.HR(Y,X))[w]
T[6] = system.time(depth.RP1(Y,X,50,15))[w]
T[7] = T[6]
T[8] = system.time({
	fX = Data2fd(argvals=t,y=t(X))
	pcX = pca.fd(fX)
	scoY = inprod(Data2fd(argvals=t,y=t(Y)),pcX$harmonics)
	depth.sample(scoY[,1],pcX$scores[,1])
	})[w]
T[9] = system.time({
	fX = Data2fd(argvals=t,y=t(X))
	pcX = pca.fd(fX)
	scoY = inprod(Data2fd(argvals=t,y=t(Y)),pcX$harmonics)
	depth.sample(scoY,pcX$scores)
	})[w]
#------
TDiff = system.time({XK = derivatives.est(X);YK = derivatives.est(Y)})[w]
T[10] = system.time(depth.fd2(YK,XK))[w]+TDiff
T[11] = T[10]
T[12] = system.time(depth.hM2(YK,XK))[w]+TDiff
T[13] = system.time(depth.BD(YK[,,2],XK[,,2]))[w]+T[4]+TDiff
T[14] = system.time(depth.HR(YK[,,2],XK[,,2]))[w]+T[5]+TDiff
T[15] = system.time(depth.RP2(YK,XK,50))[w]+TDiff
T[16] = T[15]
#-------
T[17] = system.time(depth.fd1(Y,X,order=2))[w]
T[18] = system.time(depth.fd1(Y,X,order=2,approx=100))[w]
T[19] = system.time(depth.fd1(Y,X,order=3,approx=100))[w]
T[20] = T[17]
T[21] = T[18]
T[22] = T[19]

Ttab = t(matrix(c(1:nD,DepthNames,sprintf("%.2f", round(T,2))),ncol=3))
colnames(Ttab) = 1:nD
rownames(Ttab) = c("No","Abbr","Time (in s)")

xT = xtable(Ttab,label="table:time",caption = "Computational times of the depths considered in the paper. The displayed values correspond to one simulation run of Example \\ref{ex:Gaussian} described in Section \\ref{section:outliers}.")

xTABT = print(xT,
	hline.after=c(0,nrow(xT)),
	sanitize.text.function=function(x) paste("$",x,"$",sep=""),
	colnames = FALSE,
	caption.placement = "top")
write(xTABT, file = "files_depth.fd/CompTime.tex")

#####################
#
# Table 1 from the main paper
#
#####################

rm(list=ls(all=TRUE))
if(!dir.exists("files_depth.fd")) warning("Check working directory")

library(abind)
library(xtable)
library(depth.fd)

ExNames = c("_Ex2","_Ex3")
DepthNames = 	paste("\\",c(	
				"FD","ID","hM","BD","HR","RP","RF","PC","PCB",
				"FDB","IDB","hMB","BDB","HRB","RPB","RPD",
				"FDD","FDA","FDT",
				"IDD","IDA","IDT"
				),sep="")
nD = length(DepthNames)
nE = length(ExNames)

DI = c(1,2,10,11,17,20)

for(i in 1:nE){
	load(paste("files_depth.fd/SimulationStudy",ExNames[i],".RData",sep=""))
	D = DT
	if(i==1) DG = D else DG = abind(DG,D,along=4)
	}
B = dim(DG)[3]
DC = DG[,1:nD,,]							# only depths, no infimal area indices
DC = aperm(DC,c(1,2,4,3))

RT = apply(-DC,2:4,rank,ties.method="max")		# ranking
TT = apply(DC,2:4,function(x) sum(x==min(x)))		# no of minimal value ties

M2 = apply(RT[dim(RT)[1],,,],1:2,mean)			# mean depth rank
S2 = apply(RT[dim(RT)[1],,,],1:2,sd)			# sd of depth rank
C2 = paste(sprintf("%.3f", round(M2,3))," (",sprintf("%.1f", round(S2,3)),")",sep="")

M3 = apply(TT,1:2,mean)						# mean nr of ties
S3 = apply(TT,1:2,sd)						# sd of nr of ties
C3 = paste(sprintf("%.3f", round(M3,3))," (",sprintf("%.1f", round(S3,3)),")",sep="")

nD = length(DI)
TAB = matrix(ncol=8,nrow=4)
TAB[,1] = c("\\multirow{2}{*}{Example~\\ref{ex:Gaussian}} &",
		"&",
		"\\hdashline \\multirow{2}{*}{Example~\\ref{ex:broken}} &",
		"&")
TAB[,2] = c("\\textbf{Rank}","\\textbf{Ties}","\\textbf{Rank}","\\textbf{Ties}")
for(j in 1:nD) {
	TAB[1,2+j] = paste("& ",sprintf("%.1f", round(M2[DI[j],1],3))," {\\scriptsize(",sprintf("%.1f", round(S2[DI[j],1],3)),")}",sep="")
	TAB[3,2+j] = paste("& ",sprintf("%.1f", round(M2[DI[j],2],3))," {\\scriptsize(",sprintf("%.1f", round(S2[DI[j],2],3)),")}",sep="")
	TAB[2,2+j] = paste("& ",sprintf("%.1f", round(M3[DI[j],1],3))," {\\scriptsize(",sprintf("%.1f", round(S3[DI[j],1],3)),")}",sep="")
	TAB[4,2+j] = paste("& ",sprintf("%.1f", round(M3[DI[j],2],3))," {\\scriptsize(",sprintf("%.1f", round(S3[DI[j],2],3)),")}",sep="")
	}
TAB = cbind(TAB,"\\\\ \n")
cat(apply(TAB,1,paste))

######################################
######################################
# 
# Revision: Additional simulation study in reply to reviewer
#
######################################
######################################

rm(list=ls(all=TRUE))
if(!dir.exists("files_depth.fd")) warning("Check working directory")
library(xtable)
library(depth.fd)
library(fda)
library(rainbow)
library(fda.usc)
library(robustbase)
library(doParallel)           						# for parallel computing
nCores = detectCores()
library(abind)
acomb5 <- function(...) abind(..., along=5)

# https://academic.oup.com/biostatistics/article-lookup/doi/10.1093/biostatistics/kxu006
source("files_depth.fd/OutGram.R")       	#	OutlierGram, source file from the Supplementary Material to Arribas-Gil and Romo (2014) 
source("files_depth.fd/OutGramAdj.R")    	#	Adjusted OutlierGram, source file from the Supplementary Material to Arribas-Gil and Romo (2014) 

######
# Simulation settings
######
stopurl = "https://cran.r-project.org/"
Tstart = Sys.time()

Models = 3			# no of models in the simulation study
nrun = 100			# no of runs for each combination of parameters
nM = 7			# no of methods used for detection
nset = c(50,100)		# sample sizes used
outset = c(0,0.05,.1)	# contamination percentage used
nn = length(nset)
on = length(outset)
Pr = array(dim=c(nM,nn,on,2))
Res = array(dim=c(nM,nn,on,2,nrun,Models))

# Covariance functions      
cov.fun=function(d,k,c){ k*exp(-c*d) }

# Contamination models
f.out1=function(t,m){            
	u<-runif(1,0,1) 
	if (u<0.5){
		f<-4*t+exp(-(t-m)^2/0.02)/sqrt(2*pi*0.01)-1.8
	}
	else{f<-4*t-exp(-(t-m)^2/0.02)/sqrt(2*pi*0.01)+1.8}
	return(f)
	}
f.out2=function(t,th){4*t+2*sin(4*(t+th)*pi)}
f.out5=function(t){30*(1-t)*t^(3/2)} 

p=50
t=seq(0,1,len=p)
d=dist(t,upper=T,diag=T)
d.matrix=as.matrix(d)

sim.data = function(M,n,out.rate){

	#Model 1
	if(M==1){
	t.cov=cov.fun(d.matrix,0.3,1/0.3) 
	mu=30*t*(1-t)^(3/2)
	}

	#Models 2 and 3
	if(M!=1){
	t.cov=cov.fun(d.matrix,1,1)  
	mu=4*t 
	}

	### Generate Data
	L=chol(t.cov)  					# Cholesky Decomposition
	e=matrix(rnorm(n*p),p,n)
	y=mu+t(L)%*%e   
	
	### Generate Outliers
	trueout<-sort(sample(1:n,n*out.rate))     

	for (i in trueout){
		e=rnorm(p)
		if (M==1) y[,i]= f.out5(t)+t(L)%*%e                     #Outliers for Model 1
		if (M==2) y[,i]=f.out1(t,runif(1,0.25,0.75))+t(L)%*%e   #Outliers for Model 2   
		if (M==3) y[,i]=f.out2(t,runif(1,0.25,0.75))+t(L)%*%e   #Outliers for Model 3                        
		}

	x=t(y)
	Datos=fds(t, y)					
	colnames(Datos$y) = 1:n				# necessary for foutliers function
	return(list(x=x,y=y,Datos=Datos,trueout=trueout))
	}

evalperf = function(O,trueout){
	return(c(length(setdiff(trueout,O)),length(setdiff(O,trueout))))
	}

############
# start of simulation
############
set.seed(20170204)
cl <- makePSOCKcluster(nCores)
registerDoParallel(cl)

for(M in 1:Models){

PrT = foreach(ite=1:nrun, .combine='acomb5', .multicombine=TRUE, .packages=
c("robustbase","fda.usc","depth.fd","fda","rainbow")) %dopar% {

	for (i2 in 1:nn) {
	for (j2 in 1:on) {	

	SD = sim.data(M,n<-nset[i2],out.rate<-outset[j2])

	################ Functional Boxplots (Sun and Genton 2011)
	out.fb=fbplot(SD$y,t,method='MBD',plot=F)$outpoint 
	Pr[1,i2,j2,] = evalperf(out.fb,SD$trueout)

	################ Robust Mahalanobis Distance (See Hydmann and Shang 2010)  
	FO.robMah<- foutliers(data=SD$Datos, method = "robMah")
	Pr[2,i2,j2,] = evalperf(FO.robMah$outliers,SD$trueout)
    
	################ Integrated Square Error method (Hyndman and Ullah (2007); See Hydmann and Shang 2010)        
	FO.HU<-foutliers(data=SD$Datos, method = "HUoutliers",order = 2, lambda = 3.29)
	Pr[3,i2,j2,] = evalperf(FO.HU$outliers,SD$trueout)

	################ OutlierGram 
	OG<-OutGram(SD$x,t,mag=F,plotting=0)
	Pr[4,i2,j2,] = evalperf(OG$shape.outliers,SD$trueout)

	################ OutlierGram factor adjustment 
	OG<-OutGramAdj(SD$x,t,mag=F,plotting=0)
	Pr[5,i2,j2,] = evalperf(OG$shape.outliers,SD$trueout)
   
	################ Shape outliers (Nagy et al.)
	SO = shape.fd.outliers(SD$x,max.order=2,output="matrix",exclude.out=FALSE)
	Pr[6,i2,j2,] = evalperf(which(SO[2,]),SD$trueout)

	################ Shape outliers (Nagy et al.) alternative fence choice
	# new methods to compare
	Do1 = depth.fd1(SD$x,SD$x)$Half_FD
	Do2 = depth.fd1(SD$x,SD$x,order=2)$Half_FD
	D1 = Do1-1/n
	D2 = Do2-1/n
	S2 = log(D1/D2)
	S2[is.na(S2)] = Inf
	Pr[7,i2,j2,] = evalperf(which(S2>=2*median(S2)),SD$trueout)

	################ SUMARY
	Pr[,i2,j2,1] = 1 - Pr[,i2,j2,1]/length(SD$trueout)
	Pr[,i2,j2,2] = Pr[,i2,j2,2]/(n-length(SD$trueout))
	
	}
	}
	
	Pr
	}

Res[,,,,,M] = PrT
print(paste('Model ', M))
}

methods=c("Fun. BP","Rob. Mah. dist.","ISE","Outliergram","Adj. Outliergram","FD2","FD2*")
dimnames(Res)[[1]] = methods
dimnames(Res)[[2]] = paste("n=",nset,sep="")
dimnames(Res)[[3]] = paste("o=",outset,sep="")
dimnames(Res)[[4]] = c("det.out.","fal.det.")
dimnames(Res)[[5]] = paste("i=",1:nrun,sep="")
dimnames(Res)[[6]] = paste("model=",1:Models,sep="")

stopCluster(cl)

Tend = Sys.time()
shell.exec(stopurl)

print(paste("Simulation study started: ", Tstart,sep=""))
print(paste("Simulation study finished: ", Tend,sep=""))

save(Res,file="files_depth.fd/OutlierDetection_Results.RData")

# load(file="files_depth.fd/OutlierDetection_Results.RData")

DPr = Res[,,,1,,]		# detection probability (prob that a true outlier is detected)
FPr = Res[,,,2,,]		# false detection probability (prob that a non-outlier is detected)

DPr.m = apply(DPr,c(1,2,3,5),mean)
DPr.sd = apply(DPr,c(1,2,3,5),sd)
FPr.m = apply(FPr,c(1,2,3,5),mean)
FPr.sd = apply(FPr,c(1,2,3,5),sd)

###################
# Results processing
###################

rnd = 3
eg = expand.grid(1:nM,1:on,1:nn)
Tab = matrix(nrow=nrow(eg),ncol=1+2*Models)
Tab[,1] = rep(methods,(nn*on))
for(j in 1:Models){
	for(i in 1:nrow(eg)){
		g = as.integer(eg[i,])
		if(is.na(DPr.sd[g[1],g[3],g[2],j])) Tab[i,1+(j-1)*2+1] = "-" else
		Tab[i,1+(j-1)*2+1] = paste(sprintf(paste("%.",rnd,"f",sep=""), round(DPr.m[g[1],g[3],g[2],j],rnd)),
" (",sprintf(paste("%.",rnd,"f",sep=""), round(DPr.sd[g[1],g[3],g[2],j],rnd)),")",sep="")
		Tab[i,1+(j-1)*2+2] = paste(sprintf(paste("%.",rnd,"f",sep=""), round(FPr.m[g[1],g[3],g[2],j],rnd)),
" (",sprintf(paste("%.",rnd,"f",sep=""), round(FPr.sd[g[1],g[3],g[2],j],rnd)),")",sep="")
		}
	}
colnames(Tab) = c("Method",rep(c("$p_c$","$p_f$"),Models))
rownames(Tab) = make.names(rep(1:nM,on*nn),unique=TRUE)
Tab

xTAB = xtable(Tab,label="table:reply",caption = "Results of the simulation study.")
align(xTAB) = "cl|cc|cc|cc"
eg = expand.grid(outset,nset)
xTABp = print(xTAB,
	hline.after=c(nrow(xTAB)),
	add.to.row = list(as.list(c(-1,0,nM*(0:(nn*on-1)))),
	c(" & \\multicolumn{2}{|c|}{Model 1} & \\multicolumn{2}{|c}{Model 2} & \\multicolumn{2}{|c}{Model 3}\\\\",
	"\\hline",
	paste("&&&&&&\\\\ \n n=",eg[,2],", c=",eg[,1]," &&&&&& \\\\ \n ",sep=""))),	
	sanitize.colnames.function=function(x)x,
	caption.placement = "top",
	include.rownames=FALSE) 

# resize table by width
# xTABp = sub("\\begin{tabular}","\\resizebox{\\textwidth}{!}{\\begin{tabular}",xTABp,fixed=TRUE)
# resize table by height
xTABp = sub("\\begin{tabular}","\\resizebox*{!}{\\dimexpr\\textheight-2\\baselineskip\\relax}{\\begin{tabular}",xTABp,fixed=TRUE)
xTABp = sub("\\end{tabular}","\\end{tabular}}",xTABp,fixed=TRUE)
xTABp = gsub("FD2*","FD2$^*$",xTABp,fixed=TRUE)

write(xTABp, file = "files_depth.fd//SimTab.tex")





